alg1a=function(lambda,n) {

# Algorithm that simulates n draws from the distribution of the 
# random variable Y=X^2, with X~Exp(lambda) 

u=runif(n)
x=-log(1-u)/lambda
y=x^2
return(y)
}

alg1b=function(lambda,n) {

# Algorithm that simulates n draws from the distribution of the 
# random variable Z=sqrt(X), with X~Exp(lambda)

u=runif(n)
x=-log(1-u)/lambda
z=sqrt(x)
return(z)
}

alg2=function(n) {

# Box-Muller algorithm. It simulates 2n draws from the N(0,1) distribution 

u1=runif(n)
u2=runif(n)
theta=2*pi*u1
r=sqrt(-2*log(u2))
x=r*cos(theta)
y=r*sin(theta)
X=c(x,y)
return(X)
}

alg3=function(a,b,n) {

# Algorithm that simulates n draws from the Weibull(a,b) distribution 

u=runif(n)
x=-log(u)
w=a*x^(1/b)
return(w)
}

alg4=function(a,b,n) {

# Algorithm that simulates n draws from the Cauchy distribution 

u=runif(n)
x=a+b*tan(pi*(u-0.5))
return(x)
}

alg5a=function(p,n) {

# Algorithm that simulates n draws from the Bernoulli(p) distribution 

u=runif(n)
x=1*(u<=p)+0*(u>p)
return(x)
}

alg5b=function(n) {

# Algorithm that simulates n draws from the distribution of the 
# discrete random variable X with probability function
# P(X=1)=0.3, P(X=2)=0.2, P(X=3)=0.2, P(X=4)=0.3

probs=c(0.3,0.2,0.2,0.3)
X=NULL
cumprobs=c(0,cumsum(probs))
for (i in 1:n) {
u=runif(1)
for (j in 1:4) {if((u>cumprobs[j])&(u<cumprobs[j+1])) {x=j}}
X=c(X,x)
}
return(X)
}

alg5c=function(probs,n) {

# Algorithm that simulates n draws from the distribution of the 
# discrete random variable X with probability function given in
# the input vector 'probs'

X=NULL
k=length(probs)
cumprobs=c(0,cumsum(probs))
for (i in 1:n) {
u=runif(1)
for (j in 1:k) {if((u>cumprobs[j])&(u<cumprobs[j+1])) {x=j}}
X=c(X,x)
}
return(X)
}


alg6=function(n) {

# Rejection algorithm that simulates n draws from the logistic distribution

X=NULL
while (length(X)<n) {
	u=runif(1)
	x=-log(1-u)
	u1=runif(1)
	if (u1<=1/((1+exp(-x))^2)) {
		u2=runif(1)
		if (u2<0.5) x=-x
		X=cbind(X,x)
	}
}
return(X)
}


alg7a=function(n) {

# Naive Monte Carlo integration

x=runif(n)
phi=sqrt(x*(1-x)*(1+x)*(2-x))
theta_hat=mean(phi)
return(theta_hat)
}

alg7b=function(n) {

# Monte Carlo integration with Importance Sampling

x=rbeta(n,3/2,3/2)
phiw=sqrt((1+x)*(2-x))*beta(3/2,3/2)
theta_hat=mean(phiw)
return(theta_hat)
}


